@@ -21,6 +21,7 @@ from __future__ import absolute_import
 
 import uuid
 from datetime import datetime
+from functools import wraps
 
 from ovsdbapp.backend.ovs_idl.idlutils import RowNotFound
 
@@ -28,12 +29,27 @@ import constants as ovnconst
 
 from handlers.base_handler import ElementNotFoundError
 
+import neutron.constants as neutron_constants
 from neutron.neutron_api_mappers import SecurityGroupMapper
 
 from ovndb.db_set_command import DbSetCommand
 import ovndb.acls as acl_lib
 
 
+def build_add_acl_command(f):
+    @wraps(f)
+    def build_command_from_dict(wrapped_self, port_group):
+        return [
+            wrapped_self.create_add_acl_command(port_group.uuid, acl_data)
+            for acl_data in f(wrapped_self, port_group)
+        ]
+    return build_command_from_dict
+
+
+class SecurityGroupException(AttributeError):
+    pass
+
+
 class OvnSecurityGroupApi(object):
 
     def __init__(self, idl):
@@ -42,7 +58,7 @@ class OvnSecurityGroupApi(object):
     def create_security_group(
             self, name, project_id=None, tenant_id=None, description=None):
         now = datetime.utcnow().isoformat()
-        pg_name = self._generate_name(name)
+        pg_name = self._generate_name_when_required(name)
         external_ids = {
             SecurityGroupMapper.OVN_SECURITY_GROUP_CREATE_TS: now,
             SecurityGroupMapper.OVN_SECURITY_GROUP_UPDATE_TS: now,
@@ -75,8 +91,12 @@ class OvnSecurityGroupApi(object):
             )
         except RowNotFound as e:
             raise ElementNotFoundError(e)
+        if sec_group.name == self.get_default_sec_group_name():
+            raise SecurityGroupException(
+                'Updating default security group not allowed.'
+            )
         now = datetime.utcnow().isoformat()
-        pg_name = self._generate_name(name)
+        pg_name = self._generate_name_when_required(name)
         external_ids = sec_group.external_ids
 
         external_ids[SecurityGroupMapper.OVN_SECURITY_GROUP_UPDATE_TS] = now
@@ -97,10 +117,10 @@ class OvnSecurityGroupApi(object):
         ).execute()
 
     @staticmethod
-    def _generate_name(name):
-        return 'ovirt-{name}-{gen_id}'.format(
-            name=name, gen_id=uuid.uuid4()
-        )
+    def _generate_name_when_required(name):
+        return 'ovirt_{name}_{gen_id}'.format(
+            name=name, gen_id=str(uuid.uuid4()).replace('-', '_')
+        ) if name != acl_lib.DEFAULT_PG_NAME else name
 
     @staticmethod
     def get_bumped_revision_number(security_group):
@@ -116,21 +136,49 @@ class OvnSecurityGroupApi(object):
             ether_type=None, ip_prefix=None, port_min=None, port_max=None,
             protocol=None
     ):
-        acl = acl_lib.create_acls(
-            security_group, direction, description=description,
-            ether_type=ether_type, ip_prefix=ip_prefix, min_port=port_min,
-            max_port=port_max, protocol=protocol
+        acl = acl_lib.create_acl(
+            security_group, direction=direction, ether_type=ether_type,
+            ip_prefix=ip_prefix, port_min=port_min, port_max=port_max,
+            protocol=protocol,  description=description
         )
 
-        acl_command = self._idl.pg_acl_add(
-            security_group.uuid, acl['direction'], acl['priority'],
-            acl['match'], acl['action'], severity='alert', name='',
-            **acl.get('external_ids')
-        )
-
-        return acl_command
+        return self.create_add_acl_command(security_group.uuid, acl)
 
     def delete_security_group_rule(
             self, port_group, direction, priority, match
     ):
         return self._idl.pg_acl_del(port_group, direction, priority, match)
+
+    def create_add_acl_command(self, pg_uuid, acl):
+        return self._idl.pg_acl_add(
+            pg_uuid, acl['direction'], acl['priority'],
+            acl['match'], acl['action'], severity='alert', name='',
+            **acl.get('external_ids', {})
+        )
+
+    @build_add_acl_command
+    def create_default_port_group_acls(self, port_group):
+        return acl_lib.create_default_port_group_acls(port_group)
+
+    @build_add_acl_command
+    def create_allow_all_egress_acls(self, port_group):
+        return acl_lib.create_default_allow_egress_acls(port_group)
+
+    def add_security_group_ports(self, security_group, port_id):
+        return self._idl.pg_add_ports(security_group, port_id)
+
+    def delete_security_group_ports(self, security_group, port_id):
+        return self._idl.pg_del_ports(security_group, port_id)
+
+    @staticmethod
+    def get_default_sec_group_name():
+        return acl_lib.DEFAULT_PG_NAME
+
+
+def only_rules_with_allowed_actions(f):
+    def filter_rules(*args):
+        return filter(
+            lambda rule: rule.action != neutron_constants.ACL_ACTION_DROP,
+            f(*args)
+        )
+    return filter_rules
