@@ -23,6 +23,7 @@ import pytest
 
 from ovsdbapp.backend.ovs_idl.idlutils import RowNotFound
 import constants as ovnconst
+from handlers.base_handler import BadRequestError
 from handlers.base_handler import ConflictError
 import neutron.constants as neutron_constants
 from neutron.neutron_api_mappers import InvalidRestData
@@ -186,22 +187,61 @@ class TestOvnNorth(object):
             SecurityGroupMapper.OVN_SECURITY_GROUP_PROJECT: tenant_id()
         }
     )
+    DEFAULT_SECURITY_GROUP = OvnSecurityGroupRow(
+        SECURITY_GROUP_ID,
+        'Default',
+        external_ids={
+            SecurityGroupMapper.OVN_SECURITY_GROUP_DESCRIPTION:
+                SECURITY_GROUP_DESCRIPTION,
+            SecurityGroupMapper.OVN_SECURITY_GROUP_CREATE_TS: '',
+            SecurityGroupMapper.OVN_SECURITY_GROUP_UPDATE_TS: '',
+            SecurityGroupMapper.OVN_SECURITY_GROUP_REV_NUMBER: '1',
+            SecurityGroupMapper.OVN_SECURITY_GROUP_TENANT: tenant_id(),
+            SecurityGroupMapper.OVN_SECURITY_GROUP_PROJECT: tenant_id()
+        }
+    )
 
     SECURITY_GROUP_RULE_ID_01 = UUID(int=1)
+    SECURITY_GROUP_RULE_ID_02 = UUID(int=2)
+    SECURITY_GROUP_RULE_ID_03 = UUID(int=3)
     SECURITY_GROUP_RULE_01_EXT_IDS = {
         SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_SEC_GROUP_ID:
-            SECURITY_GROUP_ID,
+            str(SECURITY_GROUP_ID),
         SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_PROTOCOL:
             neutron_constants.PROTO_NAME_TCP,
         SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_ETHERTYPE:
             neutron_constants.IPV4_ETHERTYPE
     }
+    SECURITY_GROUP_RULE_02_EXT_IDS = {
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_SEC_GROUP_ID:
+            str(SECURITY_GROUP_ID),
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_PROTOCOL: 'udp',
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_ETHERTYPE: 'IPv6'
+    }
+    SECURITY_GROUP_RULE_03_EXT_IDS = {
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_SEC_GROUP_ID:
+            str(SECURITY_GROUP_ID),
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_PROTOCOL: 'udp',
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_ETHERTYPE: 'IPv4',
+        SecurityGroupRuleMapper.OVN_SEC_GROUP_RULE_MIN_PORT: '161'
+    }
     SECURITY_GROUP_RULE_01 = OvnSecurityGroupRuleRow(
         SECURITY_GROUP_RULE_ID_01,
-        'from-lport', 'ip4 && tcp', 1001, SECURITY_GROUP_ID,
+        'from-lport', 'ip4 && tcp', 1001, SECURITY_GROUP_ID, 'allow',
         SECURITY_GROUP_RULE_01_EXT_IDS
     )
 
+    SECURITY_GROUP_RULE_02 = OvnSecurityGroupRuleRow(
+        SECURITY_GROUP_RULE_ID_02,
+        'to-lport', 'ip6 && udp', 1000, SECURITY_GROUP_ID, 'drop',
+        SECURITY_GROUP_RULE_02_EXT_IDS
+    )
+    SECURITY_GROUP_RULE_03 = OvnSecurityGroupRuleRow(
+        SECURITY_GROUP_RULE_ID_03,
+        'from-lport', 'ip4 && udp && udp.dst == 161', 1001, SECURITY_GROUP_ID,
+        'allow', SECURITY_GROUP_RULE_03_EXT_IDS
+    )
+
     ports = [PORT_1, PORT_2]
 
     networks = [NETWORK_10, NETWORK_11, NETWORK_LOCALNET_12]
@@ -657,7 +697,7 @@ class TestOvnNorth(object):
             False
         )
 
-        assert mock_db_set.call_count == 3
+        assert mock_db_set.call_count == 4
 
         assert mock_db_set.mock_calls[0] == mock.call(
             ovn_north.idl,
@@ -703,7 +743,6 @@ class TestOvnNorth(object):
             ovnconst.TABLE_LSP,
             TestOvnNorth.PORT_ID01,
             (ovnconst.ROW_LSP_DHCPV4_OPTIONS, TestOvnNorth.SUBNET_ID101),
-            (ovnconst.ROW_LSP_PORT_SECURITY, []),
             (
                 ovnconst.ROW_LSP_ADDRESSES,
                 ['{mac_address} {ip_address}'.format(
@@ -711,6 +750,7 @@ class TestOvnNorth(object):
                     ip_address=TestOvnNorth.PORT_NAME01_FIXED_IP
                 )]
             ),
+            (ovnconst.ROW_LSP_PORT_SECURITY, []),
         )
 
     @mock.patch(
@@ -1234,6 +1274,10 @@ class TestOvnNorth(object):
             )
         )
 
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAclAddCommand.execute',
+        lambda cmd, check_error: []
+    )
     @mock.patch(
         'ovsdbapp.schema.ovn_northbound.commands.PgAddCommand.execute',
         lambda cmd, check_error: TestOvnNorth.SECURITY_GROUP
@@ -1252,6 +1296,10 @@ class TestOvnNorth(object):
         )
         assert_security_group_equal(result, security_group)
 
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAclAddCommand.execute',
+        lambda cmd, check_error: []
+    )
     @mock.patch(
         'ovsdbapp.schema.ovn_northbound.commands.PgAddCommand.execute',
         lambda cmd, check_error: TestOvnNorth.SECURITY_GROUP
@@ -1299,6 +1347,10 @@ class TestOvnNorth(object):
                 == invalid_data.value.message
         )
 
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup',
+        lambda idl, table, uuid: TestOvnNorth.SECURITY_GROUP
+    )
     @mock.patch('ovsdbapp.schema.ovn_northbound.commands.PgDelCommand',
                 autospec=False)
     def test_delete_security_group(self, mock_pg_del_command, mock_connection):
@@ -1315,6 +1367,10 @@ class TestOvnNorth(object):
         'ovsdbapp.backend.ovs_idl.command.DbSetCommand',
         autospec=False
     )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAclAddCommand.execute',
+        lambda cmd, check_error: []
+    )
     @mock.patch(
         'ovsdbapp.backend.ovs_idl.command.DbListCommand.execute',
         lambda command, check_error: []
@@ -1349,6 +1405,25 @@ class TestOvnNorth(object):
 
         assert mock_db_update.call_count == 1
 
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup',
+        lambda self, table, the_id: TestOvnNorth.DEFAULT_SECURITY_GROUP
+    )
+    def test_update_default_sec_group(self, mock_connection):
+        ovn_north = NeutronApi()
+
+        # update the default security group
+        new_name = 'new_way_cuter_name'
+        new_description = 'this thing is waaaay better now'
+
+        with pytest.raises(BadRequestError):
+            ovn_north.update_security_group(
+                SecurityGroupApiInputMaker(
+                    new_name, description=new_description
+                ).get(),
+                TestOvnNorth.SECURITY_GROUP_ID
+            )
+
     @mock.patch(
         'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup',
         lambda idl, table, uuid: TestOvnNorth.SECURITY_GROUP_RULE_01
@@ -1419,7 +1494,102 @@ class TestOvnNorth(object):
         )
         assert mock_delete_rule.call_count == 1
         expected_del_call = mock.call(
-            ovn_north.idl, TestOvnNorth.SECURITY_GROUP_ID, 'from-lport', 1001,
-            'ip4 && tcp'
+            ovn_north.idl, str(TestOvnNorth.SECURITY_GROUP_ID), 'from-lport',
+            1001, 'ip4 && tcp'
         )
         assert mock_delete_rule.mock_calls[0] == expected_del_call
+
+    @mock.patch(
+        'ovsdbapp.backend.ovs_idl.command.DbListCommand.execute',
+        lambda command, check_error: [
+            TestOvnNorth.SECURITY_GROUP_RULE_01,
+            TestOvnNorth.SECURITY_GROUP_RULE_02
+        ]
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup',
+        lambda idl, table, uuid: TestOvnNorth.SECURITY_GROUP
+    )
+    def test_filter_out_drop_security_rules(self, mock_connection):
+        ovn_north = NeutronApi()
+        result = ovn_north.get_security_group(
+            TestOvnNorth.SECURITY_GROUP_ID
+        )
+        assert_security_group_equal(
+            result, SecurityGroup(
+                sec_group=TestOvnNorth.SECURITY_GROUP,
+                sec_group_rules=[TestOvnNorth.SECURITY_GROUP_RULE_01]
+            )
+        )
+
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAclAddCommand.execute'
+    )
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.commands.PgAddCommand.execute',
+        lambda cmd, check_error: TestOvnNorth.SECURITY_GROUP
+    )
+    def test_add_security_group_automatic_rule_install(
+            self, mock_ovn_getter, mock_connection
+    ):
+        ovn_north = NeutronApi()
+        mock_ovn_getter.side_effect = [
+            TestOvnNorth.SECURITY_GROUP_RULE_01,
+            TestOvnNorth.SECURITY_GROUP_RULE_03
+        ]
+        rest_data = SecurityGroupApiInputMaker(
+            TestOvnNorth.SECURITY_GROUP_NAME, tenant_id(), tenant_id(),
+            description=TestOvnNorth.SECURITY_GROUP_DESCRIPTION
+        ).get()
+
+        result = ovn_north.add_security_group(rest_data)
+        security_group = SecurityGroup(
+            sec_group=TestOvnNorth.SECURITY_GROUP,
+            sec_group_rules=[
+                TestOvnNorth.SECURITY_GROUP_RULE_01,
+                TestOvnNorth.SECURITY_GROUP_RULE_03
+            ]
+        )
+        assert_security_group_equal(result, security_group)
+
+    @mock.patch(
+        'ovsdbapp.schema.ovn_northbound.impl_idl.OvnNbApiIdlImpl.lookup'
+    )
+    def test_cannot_delete_assigned_sec_group(
+            self, mock_lookup, mock_connection
+    ):
+        security_group_id = UUID(int=101)
+        port_01_id = UUID(int=4)
+
+        port_01 = OvnPortRow(
+            port_01_id,
+            external_ids={
+                PortMapper.OVN_NIC_NAME: TestOvnNorth.PORT_NAME01,
+                PortMapper.OVN_DEVICE_ID: str(port_01_id),
+                PortMapper.OVN_DEVICE_OWNER: TestOvnNorth.DEVICE_OWNER_OVIRT,
+                PortMapper.OVN_SECURITY_GROUPS: str(security_group_id)
+            }
+        )
+
+        security_group_with_ports = OvnSecurityGroupRow(
+            security_group_id,
+            TestOvnNorth.SECURITY_GROUP_NAME,
+            external_ids={
+                SecurityGroupMapper.OVN_SECURITY_GROUP_DESCRIPTION:
+                    '',
+                SecurityGroupMapper.OVN_SECURITY_GROUP_CREATE_TS: '',
+                SecurityGroupMapper.OVN_SECURITY_GROUP_UPDATE_TS: '',
+                SecurityGroupMapper.OVN_SECURITY_GROUP_REV_NUMBER: '1',
+                SecurityGroupMapper.OVN_SECURITY_GROUP_TENANT: tenant_id(),
+                SecurityGroupMapper.OVN_SECURITY_GROUP_PROJECT: tenant_id()
+            },
+            ports=[port_01]
+        )
+
+        mock_lookup.return_value = security_group_with_ports
+        ovn_north = NeutronApi()
+        with pytest.raises(BadRequestError) as ex:
+            ovn_north.delete_security_group(security_group_id)
+        assert 'Security Group {} in use'.format(
+            security_group_id
+        ) == ex.value.message
